@@ -22,7 +22,9 @@
 
 #include <endian.h>
 #include "nls.h"
-#include <linux/major.h>        /* FLOPPY_MAJOR */
+#include "xalloc.h"
+
+#include "blkdev.h"
 
 #include "common.h"
 #include "fdisk.h"
@@ -41,8 +43,10 @@ static freeblocks freelist[17]; /* 16 partitions can produce 17 vacant slots */
 
 static void
 setfreelist(int i, unsigned int f, unsigned int l) {
-	freelist[i].first = f;
-	freelist[i].last = l;
+	if (i < 17) {
+		freelist[i].first = f;
+		freelist[i].last = l;
+	}
 }
 
 static void
@@ -98,19 +102,6 @@ struct systypes sgi_sys_types[] = {
 	{0, NULL }
 };
 
-static inline unsigned short
-__swap16(unsigned short x) {
-        return (((__u16)(x) & 0xFF) << 8) | (((__u16)(x) & 0xFF00) >> 8);
-}
-
-static inline __u32
-__swap32(__u32 x) {
-        return (((x & 0xFF) << 24) |
-		((x & 0xFF00) << 8) |
-		((x & 0xFF0000) >> 8) |
-		((x & 0xFF000000) >> 24));
-}
-
 static int
 sgi_get_nsect(void) {
 	return SSWAP16(sgilabel->devparam.nsect);
@@ -197,7 +188,7 @@ sgi_list_table(int xtra) {
 	w = strlen(disk_device);
 
 	if (xtra) {
-		printf(_("\nDisk %s (SGI disk label): %d heads, %d sectors\n"
+		printf(_("\nDisk %s (SGI disk label): %d heads, %llu sectors\n"
 			 "%d cylinders, %d physical cylinders\n"
 			 "%d extra sects/cyl, interleave %d:1\n"
 			 "%s\n"
@@ -211,7 +202,7 @@ sgi_list_table(int xtra) {
                        sector_size);
 	} else {
 		printf(_("\nDisk %s (SGI disk label): "
-			 "%d heads, %d sectors, %d cylinders\n"
+			 "%d heads, %llu sectors, %d cylinders\n"
 			 "Units = %s of %d * %d bytes\n\n"),
 		       disk_device, heads, sectors, cylinders,
 		       str_units(PLURAL), units_per_sector,
@@ -222,8 +213,8 @@ sgi_list_table(int xtra) {
 	       w + 1, _("Device"));
 	for (i = 0 ; i < partitions; i++) {
 		if (sgi_get_num_sectors(i) || debug) {
-			__u32 start = sgi_get_start_sector(i);
-			__u32 len = sgi_get_num_sectors(i);
+			uint32_t start = sgi_get_start_sector(i);
+			uint32_t len = sgi_get_num_sectors(i);
 			kpi++;		/* only count nonempty partitions */
 			printf(
 				"%2d: %s %4s %9ld %9ld %9ld  %2x  %s\n",
@@ -244,9 +235,9 @@ sgi_list_table(int xtra) {
 	       sgilabel->boot_file);
 	for (i = 0 ; i < volumes; i++) {
 		if (sgilabel->directory[i].vol_file_size) {
-			__u32 start = SSWAP32(sgilabel->directory[i].vol_file_start);
-			__u32 len = SSWAP32(sgilabel->directory[i].vol_file_size);
-			char *name = sgilabel->directory[i].vol_file_name;
+			uint32_t start = SSWAP32(sgilabel->directory[i].vol_file_start);
+			uint32_t len = SSWAP32(sgilabel->directory[i].vol_file_size);
+			unsigned char *name = sgilabel->directory[i].vol_file_name;
 			printf(_("%2d: %-10s sector%5u size%8u\n"),
 			       i, name, (unsigned int) start,
 			       (unsigned int) len);
@@ -318,7 +309,7 @@ sgi_check_bootfile(const char* aFile) {
 			}
 		}
 	}
-	if (strncmp(aFile, sgilabel->boot_file, 16)) {
+	if (strncmp(aFile, (char *) sgilabel->boot_file, 16)) {
 		printf(_("\n\tBe aware, that the bootfile is not checked for existence.\n\t"
 			 "SGI's default is \"/unix\" and for backup \"/unix.save\".\n"));
 		/* filename is correct and did change */
@@ -329,7 +320,7 @@ sgi_check_bootfile(const char* aFile) {
 
 const char *
 sgi_get_bootfile(void) {
-	return sgilabel->boot_file;
+	return (char *) sgilabel->boot_file;
 }
 
 void
@@ -355,7 +346,7 @@ create_sgiinfo(void) {
 	/* I keep SGI's habit to write the sgilabel to the second block */
 	sgilabel->directory[0].vol_file_start = SSWAP32(2);
 	sgilabel->directory[0].vol_file_size = SSWAP32(sizeof(sgiinfo));
-	strncpy(sgilabel->directory[0].vol_file_name, "sgilabel", 8);
+	strncpy((char *) sgilabel->directory[0].vol_file_name, "sgilabel", 8);
 }
 
 sgiinfo *fill_sgiinfo(void);
@@ -372,14 +363,14 @@ sgi_write_table(void) {
 		fatal(unable_to_seek);
 	if (write(fd, sgilabel, SECTOR_SIZE) != SECTOR_SIZE)
 		fatal(unable_to_write);
-	if (! strncmp(sgilabel->directory[0].vol_file_name, "sgilabel", 8)) {
+	if (! strncmp((char *) sgilabel->directory[0].vol_file_name, "sgilabel", 8)) {
 		/*
 		 * keep this habit of first writing the "sgilabel".
 		 * I never tested whether it works without (AN 981002).
 		 */
 		sgiinfo *info = fill_sgiinfo();
 		int infostartblock = SSWAP32(sgilabel->directory[0].vol_file_start);
-		if (ext2_llseek(fd, (long long)infostartblock*
+		if (lseek(fd, (off_t) infostartblock*
 				SECTOR_SIZE, SEEK_SET) < 0)
 			fatal(unable_to_seek);
 		if (write(fd, info, SECTOR_SIZE) != SECTOR_SIZE)
@@ -501,6 +492,9 @@ verify_sgi(int verbose)
 		}
 		start = sgi_get_start_sector(Index[i])
 			+ sgi_get_num_sectors(Index[i]);
+		/* Align free space on cylinder boundary */
+		if (start % cylsize)
+			start += cylsize - (start % cylsize);
 		if (debug > 1) {
 			if (verbose)
 				printf("%2d:%12d\t%12d\t%12d\n", Index[i],
@@ -537,12 +531,12 @@ verify_sgi(int verbose)
 	return (gap > 0) ? 1 : (gap == 0) ? 0 : -1;
 }
 
-void
+int
 sgi_change_sysid(int i, int sys)
 {
 	if (sgi_get_num_sectors(i) == 0) /* caught already before, ... */ {
 		printf(_("Sorry You may change the Tag of non-empty partitions.\n"));
-		return;
+		return 0;
 	}
 	if (((sys != ENTIRE_DISK) && (sys != SGI_VOLHDR))
 	    && (sgi_get_start_sector(i)<1)) {
@@ -553,9 +547,10 @@ sgi_change_sysid(int i, int sys)
 			  "Only the \"SGI volume\" entire disk section may violate this.\n"
 			  "Type YES if you are sure about tagging this partition differently.\n"));
 		if (strcmp (line_ptr, _("YES\n")))
-			return;
+			return 0;
 	}
 	sgilabel->partitions[i].id = SSWAP32(sys);
+	return 1;
 }
 
 /* returns partition index of first entry marked as entire disk */
@@ -709,8 +704,9 @@ create_sgilabel(void)
 
 	other_endian = (BYTE_ORDER == LITTLE_ENDIAN);
 
-	res = disksize(fd, &llsectors);
+	res = blkdev_get_sectors(fd, &llsectors);
 
+#ifdef HDIO_GETGEO
 	if (!ioctl(fd, HDIO_GETGEO, &geometry)) {
 		heads = geometry.heads;
 		sectors = geometry.sectors;
@@ -731,6 +727,7 @@ create_sgilabel(void)
 				  " > 33.8 GB.\n"), disk_device, cylinders);
 		}
 	}
+#endif
 	for (i = 0; i < 4; i++) {
 		old[i].sysid = 0;
 		if (valid_part_table_flag(MBRbuffer)) {
@@ -746,14 +743,14 @@ create_sgilabel(void)
 		}
 	}
 
-	memset(MBRbuffer, 0, sizeof(MBRbuffer));
+	zeroize_mbr_buffer();
 	sgilabel->magic = SSWAP32(SGI_LABEL_MAGIC);
 	sgilabel->boot_part = SSWAP16(0);
 	sgilabel->swap_part = SSWAP16(1);
 
 	/* sizeof(sgilabel->boot_file) = 16 > 6 */
 	memset(sgilabel->boot_file, 0, 16);
-	strcpy(sgilabel->boot_file, "/unix");
+	strcpy((char *) sgilabel->boot_file, "/unix");
 
 	sgilabel->devparam.skew			= (0);
 	sgilabel->devparam.gap1			= (0);
@@ -831,15 +828,15 @@ sgi_set_ncyl(void)
 sgiinfo *
 fill_sgiinfo(void)
 {
-	sgiinfo*info=calloc(1, sizeof(sgiinfo));
+	sgiinfo *info=xcalloc(1, sizeof(sgiinfo));
 	info->magic=SSWAP32(SGI_INFO_MAGIC);
 	info->b1=SSWAP32(-1);
 	info->b2=SSWAP16(-1);
 	info->b3=SSWAP16(1);
 	/* You may want to replace this string !!!!!!! */
-	strcpy(info->scsi_string, "IBM OEM 0662S12         3 30");
-	strcpy(info->serial, "0000");
+	strcpy((char *) info->scsi_string, "IBM OEM 0662S12         3 30");
+	strcpy((char *) info->serial, "0000");
 	info->check1816 = SSWAP16(18*256 +16);
-	strcpy(info->installer, "Sfx version 5.3, Oct 18, 1994");
+	strcpy((char *) info->installer, "Sfx version 5.3, Oct 18, 1994");
 	return info;
 }
