@@ -71,7 +71,7 @@ struct libmnt_context *mnt_new_context(void)
 	mnt_has_regular_mtab(&cxt->mtab_path, &cxt->mtab_writable);
 
 	if (!cxt->mtab_writable)
-		/* use /dev/.mount/utab if /etc/mtab is useless */
+		/* use /run/mount/utab if /etc/mtab is useless */
 		mnt_has_regular_utab(&cxt->utab_path, &cxt->utab_writable);
 
 	return cxt;
@@ -794,13 +794,21 @@ struct libmnt_cache *mnt_context_get_cache(struct libmnt_context *cxt)
  * mnt_context_get_lock:
  * @cxt: mount context
  *
- * The application that uses libmount context does not have to care about
- * mtab locking, but with a small exceptions: the application has to be able to
- * remove the lock file when interrupted by signal. It means that properly written
- * mount(8)-like application has to call mnt_unlock_file() from a signal handler.
+ * The libmount applications don't have to care about mtab locking, but with a
+ * small exception: the application has to be able to remove the lock file when
+ * interrupted by signal or signals have to be ignored when the lock is locked.
  *
- * This function returns NULL if mtab file is not writable or nolock or nomtab
- * flags is enabled.
+ * The default behavior is to ignore all signals (except SIGALRM and SIGTRAP)
+ * when the lock is locked. If this behavior is unacceptable then use:
+ *
+ *	lc = mnt_context_get_lock(cxt);
+ *	if (lc)
+ *		mnt_lock_block_signals(lc, FALSE);
+ *
+ * and don't forget to call mnt_unlock_file(lc) before exit.
+ *
+ * This function returns NULL if the lock is unnecessary (mtab file is not writable
+ * or /etc/mtab is symlink to /proc/mounts).
  *
  * Returns: pointer to lock struct or NULL.
  */
@@ -809,9 +817,11 @@ struct libmnt_lock *mnt_context_get_lock(struct libmnt_context *cxt)
 	if (!cxt || (cxt->flags & MNT_FL_NOMTAB) || !cxt->mtab_writable)
 		return NULL;
 
-	if (!cxt->lock && cxt->mtab_path)
+	if (!cxt->lock && cxt->mtab_path) {
 		cxt->lock = mnt_new_lock(cxt->mtab_path, 0);
-
+		if (cxt->lock)
+			mnt_lock_block_signals(cxt->lock, TRUE);
+	}
 	return cxt->lock;
 }
 
@@ -1218,7 +1228,7 @@ int mnt_context_merge_mflags(struct libmnt_context *cxt)
 }
 
 /*
- * Prepare /etc/mtab or /dev/.mount/utab
+ * Prepare /etc/mtab or /run/mount/utab
  */
 int mnt_context_prepare_update(struct libmnt_context *cxt)
 {
@@ -1481,7 +1491,7 @@ int mnt_context_apply_fstab(struct libmnt_context *cxt)
  * mnt_context_get_status:
  * @cxt: mount context
  *
- * Returns: 0 if /sbin/mount.type or mount(2) syscall was successfull or -errno.
+ * Returns: 0 if /sbin/mount.type or mount(2) syscall was successfull.
  */
 int mnt_context_get_status(struct libmnt_context *cxt)
 {
@@ -1493,6 +1503,9 @@ int mnt_context_get_status(struct libmnt_context *cxt)
  * @cxt: mount context
  * @status: mount(2) return code
  *
+ * The @status should be 0 on succcess, or negative number on error (-1 or
+ * -errno).
+ *
  * This function should be used if [u]mount(2) syscall was NOT called by
  * libmount (by mnt_context_mount() or mnt_context_do_mount()) only.
  *
